@@ -168,34 +168,38 @@ const char *apply_namespace(const char *mbox, char *prefix, char delim);
 const char *reverse_namespace(const char *mbox, char *prefix, char delim);
 
 /*	request.c	*/
-int request_noop(const char *server, const char *user);
-int request_login(const char *server, const char *user, const char *pass,
-    const char *port, const char *ssl);
-int request_logout(const char *server, const char *user);
-int request_status(const char *server, const char *user, const char *mbox,
-    unsigned int *exist, unsigned int *recent, unsigned int *unseen);
-int request_select(const char *server, const char *user, const char *mbox);
-int request_close(const char *server, const char *user);
-int request_list(const char *server, const char *user, const char *refer,
-    const char *name, char **mboxs, char **folders);
-int request_lsub(const char *server, const char *user, const char *refer,
-    const char *name, char **mboxs, char **folders);
-int request_search(const char *server, const char *user, const char *criteria,
-    char **mesgs);
-int request_fetchfast(const char *server, const char *user, const char *mesg,
-    char **flags, char **date, char **size);
-int request_fetchheader(const char *server, const char *user, const char *mesg,
-    char **header, size_t *len);
-int request_fetchtext(const char *server, const char *user, const char *mesg,
-    char **text, size_t *len);
-int request_fetchfields(const char *server, const char *user,
-    const char *mesg, const char *headerfields, char **fields, size_t *len);
-int request_store(const char *server, const char *user, const char *mesg,
-    const char *mode, const char *flags);
-int request_copy(const char *server, const char *user, const char *mesg,
+int request_noop(const char *server, const char *port, const char *user);
+int request_login(const char *server, const char *port, const char *ssl,
+    const char *user, const char *pass);
+int request_logout(const char *server, const char *port, const char *user);
+int request_status(const char *server, const char *port, const char *user,
+    const char *mbox, unsigned int *exist, unsigned int *recent,
+    unsigned int *unseen);
+int request_select(const char *server, const char *port, const char *user,
     const char *mbox);
-int request_append(const char *server, const char *user, const char *mbox,
-    const char *mesg, size_t mesglen, const char *flags, const char *date);
+int request_close(const char *server, const char *port, const char *user);
+int request_expunge(const char *server, const char *port, const char *user);
+int request_list(const char *server, const char *port, const char *user,
+    const char *refer, const char *name, char **mboxs, char **folders);
+int request_lsub(const char *server, const char *port, const char *user,
+    const char *refer, const char *name, char **mboxs, char **folders);
+int request_search(const char *server, const char *port, const char *user,
+    const char *criteria, char **mesgs);
+int request_fetchfast(const char *server, const char *port, const char *user,
+    const char *mesg, char **flags, char **date, char **size);
+int request_fetchheader(const char *server, const char *port, const char *user,
+    const char *mesg, char **header, size_t *len);
+int request_fetchtext(const char *server, const char *port, const char *user,
+    const char *mesg, char **text, size_t *len);
+int request_fetchfields(const char *server, const char *port, const char *user,
+    const char *mesg, const char *headerfields, char **fields, size_t *len);
+int request_store(const char *server, const char *port, const char *user,
+    const char *mesg, const char *mode, const char *flags);
+int request_copy(const char *server, const char *port, const char *user,
+    const char *mesg, const char *mbox);
+int request_append(const char *server, const char *port, const char *user,
+    const char *mbox, const char *mesg, size_t mesglen, const char *flags,
+    const char *date);
 
 /*	response.c	*/
 int response_generic(session *ssn, int tag);
