@@ -165,7 +165,20 @@ function flag_aux(account, mbox, mode, flags, messages)
 
     local f = ''
     if (table.getn(flags) ~= 0) then
-	f = '\\' .. table.concat(flags, ' \\')
+	if (flags.keywords ~= true) then
+	    for k, v in ipairs(flags) do
+		if (string.lower(v) == 'answered' or
+		    string.lower(v) == 'deleted' or
+		    string.lower(v) == 'draft' or
+		    string.lower(v) == 'flagged' or
+		    string.lower(v) == 'seen') then
+		    f = f .. '\\' .. v .. ' '
+		end
+	    end
+	    f = string.gsub(f, '(.+) ', '%1')
+	else
+	    f = table.concat(flags, ' ')
+	end
     end
 
     local m = range_sequence(messages)
@@ -260,16 +273,13 @@ function copy_aux(srcaccount, srcmbox, dstaccount, dstmbox, messages)
 
 	for i in pairs(fast) do
 	    for k, v in ipairs(fast[i]['flags']) do
-		if (string.lower(v) == 'recent') then
+		if (string.lower(v) == '\\recent') then
 		    table.remove(fast[i]['flags'], k)
 		end
 	    end
 
-	    local f = ''
-	    if (table.getn(fast[i]['flags']) ~= 0) then
-		f = '\\' .. table.concat(fast[i]['flags'], ' \\')
-	    end
-	    r = ifcore.append(dstaccount, dstmbox, msgs[i], f, fast[i]['date'])
+	    r = ifcore.append(dstaccount, dstmbox, msgs[i],
+	        table.concat(fast[i]['flags'], ' '), fast[i]['date'])
 	end
     end
 
@@ -524,7 +534,7 @@ function fetchfast(account, mbox, messages)
 	local _, flags, date, size = ifcore.fetchfast(account, tostring(v))
 	if (flags ~= nil and date ~= nil and size ~= nil ) then
 	    local f = {}
-	    for s in string.gfind(flags, '%w+') do
+	    for s in string.gfind(flags, '%S+') do
 		table.insert(f, s)
 	    end
 	    results[tonumber(v)] = {}
@@ -571,13 +581,15 @@ function list(account, name)
     local _, mboxs, folders = ifcore.list(account, '', name .. '%')
     
     local m = {}
-    for s in string.gfind(mboxs, '[%w%p]+') do
+    for s in string.gfind(mboxs, '%C+') do
 	table.insert(m, s)
     end
 
     local f = {}
-    for s in string.gfind(folders, '[%w%p]+') do
-	table.insert(f, s)
+    for s in string.gfind(folders, '%C+') do
+	if s ~= name and s ~= name .. '/' then
+	    table.insert(f, s)
+	end
     end
 
     return m, f
@@ -613,13 +625,15 @@ function lsub(account, name)
     local _, mboxs, folders = ifcore.lsub(account, '', name .. '%')
     
     local m = {}
-    for s in string.gfind(mboxs, '[%w%p]+') do
+    for s in string.gfind(mboxs, '%C+') do
 	table.insert(m, s)
     end
 
     local f = {}
-    for s in string.gfind(folders, '[%w%p]+') do
-	table.insert(f, s)
+    for s in string.gfind(folders, '%C+') do
+	if s ~= name and s ~= name .. '/' then
+	    table.insert(f, s)
+	end
     end
 
     return m, f
