@@ -29,14 +29,21 @@ void
 start_lua()
 {
 
+#if LUA_VERSION_NUM >= 501
+	lua = luaL_newstate();
+
+	luaL_openlibs(lua);
+#else
 	lua = lua_open();
 
 	luaopen_base(lua);
-	luaopen_string(lua);
 	luaopen_table(lua);
-	luaopen_math(lua);
 	luaopen_io(lua);
+	luaopen_string(lua);
+	luaopen_math(lua);
 	luaopen_debug(lua);
+	luaopen_loadlib(lua);
+#endif
 
 	luaopen_ifcore(lua);
 	luaopen_ifsys(lua);
