@@ -17,12 +17,12 @@ int create_mailbox(session *ssn, const char *mbox);
  * Reset any inactivity autologout timer on the server.
  */
 int
-request_noop(const char *server, const char *user)
+request_noop(const char *server, const char *port, const char *user)
 {
 	int r;
 	session *s;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	if ((r = response_generic(s, imap_noop(s))) == -1)
@@ -42,26 +42,26 @@ fail:
  * the namespace of the mailboxes.
  */
 int
-request_login(const char *server, const char *user, const char *pass,
-    const char *port, const char *ssl)
+request_login(const char *server, const char *port, const char *ssl,
+    const char *user, const char *pass)
 {
 	int r, rg;
 	session *s;
 
-	if ((s = session_find(server, user)))
+	if ((s = session_find(server, port, user)))
 		return STATUS_RESPONSE_OK;
 
 	s = session_new();
 
 	s->server = xstrdup(server);
+	s->port = xstrdup(port);
 	s->username = xstrdup(user);
 
 	if (ssl && strncasecmp(ssl, "tls1", 4) &&
 	    strncasecmp(ssl, "ssl3", 4) && strncasecmp(ssl, "ssl2", 4))
 		ssl = NULL;
 
-	if (open_connection(s, server, (port ? port : (!ssl ? "143" : "993")),
-	    ssl) == -1)
+	if (open_connection(s, server, port, ssl) == -1)
 		goto fail;
 
 	if ((rg = response_greeting(s)) == -1)
@@ -129,12 +129,12 @@ fail:
  * Logout from the IMAP server and disconnect from the server.
  */
 int
-request_logout(const char *server, const char *user)
+request_logout(const char *server, const char *port, const char *user)
 {
 	int r;
 	session *s;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	r = response_generic(s, imap_logout(s));
@@ -150,14 +150,15 @@ request_logout(const char *server, const char *user)
  * Get mailbox's status.
  */
 int
-request_status(const char *server, const char *user, const char *mbox,
-    unsigned int *exists, unsigned int *recent, unsigned int *unseen)
+request_status(const char *server, const char *port, const char *user,
+    const char *mbox, unsigned int *exists, unsigned int *recent,
+    unsigned int *unseen)
 {
 	int t, r;
 	session *s;
 	const char *m;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
@@ -187,13 +188,14 @@ fail:
  * Open mailbox in read-write mode.
  */
 int
-request_select(const char *server, const char *user, const char *mbox)
+request_select(const char *server, const char *port, const char *user,
+    const char *mbox)
 {
 	int r;
 	session *s;
 	const char *m;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
@@ -214,12 +216,12 @@ fail:
  * Close examined/selected mailbox.
  */
 int
-request_close(const char *server, const char *user)
+request_close(const char *server, const char *port, const char *user)
 {
 	int r;
 	session *s;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	if ((r = response_generic(s, imap_close(s))) == -1)
@@ -234,18 +236,42 @@ fail:
 }
 
 
+/*
+ * Remove all messages marked for deletion from selected mailbox.
+ */
+int
+request_expunge(const char *server, const char *port, const char *user)
+{
+	int r;
+	session *s;
+
+	if (!(s = session_find(server, port, user)))
+		return -1;
+
+	if ((r = response_generic(s, imap_expunge(s))) == -1)
+		goto fail;
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
+	return -1;
+}
+
+
 /*
  * List available mailboxes.
  */
 int
-request_list(const char *server, const char *user, const char *refer,
-    const char *name, char **mboxs, char **folders)
+request_list(const char *server, const char *port, const char *user,
+    const char *refer, const char *name, char **mboxs, char **folders)
 {
 	int t, r;
 	session *s;
 	const char *n;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	n = apply_namespace(name, s->ns.prefix, s->ns.delim);
@@ -267,14 +293,14 @@ fail:
  * List subscribed mailboxes.
  */
 int
-request_lsub(const char *server, const char *user, const char *refer,
-    const char *name, char **mboxs, char **folders)
+request_lsub(const char *server, const char *port, const char *user,
+    const char *refer, const char *name, char **mboxs, char **folders)
 {
 	int t, r;
 	session *s;
 	const char *n;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	n = apply_namespace(name, s->ns.prefix, s->ns.delim);
@@ -296,13 +322,13 @@ fail:
  * Search selected mailbox according to the supplied search criteria.
  */
 int
-request_search(const char *server, const char *user, const char *criteria,
-    char **mesgs)
+request_search(const char *server, const char *port, const char *user,
+    const char *criteria, char **mesgs)
 {
 	int t, r;
 	session *s;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	t = imap_search(s, get_option_boolean("uid"),
@@ -323,13 +349,13 @@ fail:
  * Fetch the FLAGS, INTERNALDATE and RFC822.SIZE of the messages.
  */
 int
-request_fetchfast(const char *server, const char *user, const char *mesg,
-    char **flags, char **date, char **size)
+request_fetchfast(const char *server, const char *port, const char *user,
+    const char *mesg, char **flags, char **date, char **size)
 {
 	int t, r;
 	session *s;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	t = imap_fetch(s, get_option_boolean("uid"), mesg, "FAST");
@@ -349,13 +375,13 @@ fail:
  * Fetch the header, ie. BODY[HEADER], of the messages.
  */
 int
-request_fetchheader(const char *server, const char *user, const char *mesg,
-    char **header, size_t *len)
+request_fetchheader(const char *server, const char *port, const char *user,
+    const char *mesg, char **header, size_t *len)
 {
 	int t, r;
 	session *s;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	t = imap_fetch(s, get_option_boolean("uid"), mesg, "BODY.PEEK[HEADER]");
@@ -375,13 +401,13 @@ fail:
  * Fetch the text, ie. BODY[TEXT], of the messages.
  */
 int
-request_fetchtext(const char *server, const char *user, const char *mesg,
-    char **text, size_t *len)
+request_fetchtext(const char *server, const char *port, const char *user,
+    const char *mesg, char **text, size_t *len)
 {
 	int t, r;
 	session *s;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	t = imap_fetch(s, get_option_boolean("uid"), mesg, "BODY.PEEK[TEXT]");
@@ -402,8 +428,8 @@ fail:
  * messages.
  */
 int
-request_fetchfields(const char *server, const char *user, const char *mesg,
-    const char *headerfields, char **fields, size_t *len)
+request_fetchfields(const char *server, const char *port, const char *user,
+    const char *mesg, const char *headerfields, char **fields, size_t *len)
 {
 	int t, r, n;
 	session *s;
@@ -413,7 +439,7 @@ request_fetchfields(const char *server, const char *user, const char *mesg,
 	f = (char *)xmalloc(n * sizeof(char));
 	snprintf(f, n, "%s%s%s", "BODY.PEEK[HEADER.FIELDS (", headerfields, ")]");
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	t = imap_fetch(s, get_option_boolean("uid"), mesg, f);
@@ -435,13 +461,13 @@ fail:
  * Add, remove or replace the specified flags of the messages.
  */
 int
-request_store(const char *server, const char *user, const char *mesg,
-    const char *mode, const char *flags)
+request_store(const char *server, const char *port, const char *user,
+    const char *mesg, const char *mode, const char *flags)
 {
 	int t, r;
 	session *s;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	t = imap_store(s, get_option_boolean("uid"), mesg, mode, flags);
@@ -465,14 +491,14 @@ fail:
  * Copy the specified messages to another mailbox.
  */
 int
-request_copy(const char *server, const char *user, const char *mesg,
-    const char *mbox)
+request_copy(const char *server, const char *port, const char *user,
+    const char *mesg, const char *mbox)
 {
 	int t, r;
 	session *s;
 	const char *m;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
@@ -503,14 +529,15 @@ fail:
  * Append supplied message to the specified mailbox.
  */
 int
-request_append(const char *server, const char *user, const char *mbox,
-    const char *mesg, size_t mesglen, const char *flags, const char *date)
+request_append(const char *server, const char *port, const char *user,
+    const char *mbox, const char *mesg, size_t mesglen, const char *flags,
+    const char *date)
 {
 	int t, r;
 	session *s;
 	const char *m;
 
-	if (!(s = session_find(server, user)))
+	if (!(s = session_find(server, port, user)))
 		return -1;
 
 	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
