@@ -11,6 +11,8 @@
 #include "regexp.h"
 
 
+extern options opts;
+
 buffer ibuf;			/* Input buffer. */
 enum {				/* Server data responses to be parsed;
 				 * regular expressions index. */
@@ -25,15 +27,12 @@ enum {				/* Server data responses to be parsed;
 	DATA_RESPONSE_EXAMINE_EXISTS,
 	DATA_RESPONSE_EXAMINE_RECENT,
 	DATA_RESPONSE_LIST,
-	DATA_RESPONSE_LSUB,
 	DATA_RESPONSE_SEARCH,
 	DATA_RESPONSE_FETCH,
 	DATA_RESPONSE_FETCH_FLAGS,
 	DATA_RESPONSE_FETCH_DATE,
 	DATA_RESPONSE_FETCH_SIZE,
-	DATA_RESPONSE_FETCH_HEADER,
 	DATA_RESPONSE_FETCH_BODY,
-	DATA_RESPONSE_FETCH_FIELDS,
 };
 regexp responses[] = {		/* Server data responses to be parsed;
 				 * regular expressions patterns. */
@@ -49,10 +48,7 @@ regexp responses[] = {		/* Server data responses to be parsed;
 	{ "UNSEEN ([[:digit:]]+)", NULL, 0, NULL },
 	{ "\\* ([[:digit:]]+) EXISTS\r\n", NULL, 0, NULL },
 	{ "\\* ([[:digit:]]+) RECENT\r\n", NULL, 0, NULL },
-	{ "\\* LIST \\(([[:print:]]*)\\) (\"[[:print:]]\"|NIL) "
-	  "(\"([[:print:]]+)\"|([[:print:]]+)|\\{([[:digit:]]+)\\}\r\n"
-	  "([[:print:]]*))\r\n", NULL, 0, NULL },
-	{ "\\* LSUB \\(([[:print:]]*)\\) (\"[[:print:]]\"|NIL) "
+	{ "\\* (LIST|LSUB) \\(([[:print:]]*)\\) (\"[[:print:]]\"|NIL) "
 	  "(\"([[:print:]]+)\"|([[:print:]]+)|\\{([[:digit:]]+)\\}\r\n"
 	  "([[:print:]]*))\r\n", NULL, 0, NULL },
 	{ "\\* SEARCH ?([[:digit:] ]*)\r\n", NULL, 0, NULL },
@@ -60,22 +56,18 @@ regexp responses[] = {		/* Server data responses to be parsed;
 	{ "FLAGS \\(([[:print:]]*)\\)", NULL, 0, NULL },
 	{ "INTERNALDATE \"([[:print:]]*)\"", NULL, 0, NULL },
 	{ "RFC822.SIZE ([[:digit:]]+)", NULL, 0, NULL },
-	{ "BODY\\[HEADER\\] \\{([[:digit:]]+)\\}\r\n([[:print:]\r\n]*)", NULL, 0,
-	  NULL },
-	{ "BODY\\[TEXT\\] (\\{([[:digit:]]+)\\}\r\n([[:print:]\r\n]*)|"
-	  "\"([[:print:]]*)\")", NULL, 0, NULL },
-	{ "BODY\\[HEADER\\.FIELDS \\([[:print:]]*\\)\\] \\{([[:digit:]]+)\\}"
-	  "\r\n([[:print:]\r\n]*)", NULL, 0, NULL },
+	{ "\\* [[:digit:]]+ FETCH \\([[:print:]]*BODY\\[[[:print:]]*\\] "
+	  "(\\{([[:digit:]]+)\\}\r\n|\"([[:print:]]*)\")", NULL, 0, NULL },
 	{ NULL, NULL, 0, NULL }
 };
 
 
 int receive_response(session *ssn, char *buf);
 
-int check_tag(session *ssn, int tag);
-int check_bye(void);
-int check_continuation(void);
-int check_trycreate(void);
+int check_tag(char *buf, session *ssn, int tag);
+int check_bye(char *buf);
+int check_continuation(char *buf);
+int check_trycreate(char *buf);
 
 
 /*
@@ -84,13 +76,24 @@ int check_trycreate(void);
 int
 receive_response(session *ssn, char *buf)
 {
+	ssize_t n;
 
-	if (socket_read(ssn, buf, INPUT_BUF) == -1)
+	if ((n = socket_read(ssn, buf, INPUT_BUF)) == -1)
 		return -1;
 
-	debug("getting response (%d):\n\n%s\n", ssn->socket, buf);
 
-	return 0;
+	if (opts.debug > 0) {
+		int i;
+		
+		debug("getting response (%d):\n\n", ssn->socket);
+
+		for (i = 0; i < n; i++)
+			debugc(buf[i]);
+
+		debug("\n");
+	}
+
+	return n;
 }
 
 
@@ -98,7 +101,7 @@ receive_response(session *ssn, char *buf)
  * Search for tagged response in the data that the server sent.
  */
 int
-check_tag(session *ssn, int tag)
+check_tag(char *buf, session *ssn, int tag)
 {
 	int r;
 	char t[4 + 1];
@@ -110,24 +113,23 @@ check_tag(session *ssn, int tag)
 
 	re = &responses[DATA_RESPONSE_TAGGED];
 
-	if (!regexec(re->preg, ibuf.data, re->nmatch, re->pmatch, 0)) {
-		if (!strncasecmp(ibuf.data + re->pmatch[1].rm_so, t,
+	if (!regexec(re->preg, buf, re->nmatch, re->pmatch, 0)) {
+		if (!strncasecmp(buf + re->pmatch[1].rm_so, t,
 		    strlen(t))) {
-			if (!strncasecmp(ibuf.data + re->pmatch[2].rm_so,
+			if (!strncasecmp(buf + re->pmatch[2].rm_so,
 			    "OK", strlen("OK")))
 				r = STATUS_RESPONSE_OK;
-			else if (!strncasecmp(ibuf.data + re->pmatch[2].rm_so,
+			else if (!strncasecmp(buf + re->pmatch[2].rm_so,
 			    "NO", strlen("NO")))
 				r = STATUS_RESPONSE_NO;
-			else if (!strncasecmp(ibuf.data + re->pmatch[2].rm_so,
+			else if (!strncasecmp(buf + re->pmatch[2].rm_so,
 			    "BAD", strlen("BAD")))
 				r = STATUS_RESPONSE_BAD;
 		}
 	}
 
 	if (r != STATUS_RESPONSE_NONE)
-		verbose("S (%d): %s", ssn->socket, ibuf.data +
-		    re->pmatch[0].rm_so);
+		verbose("S (%d): %s", ssn->socket, buf + re->pmatch[0].rm_so);
 
 	return r;
 }
@@ -137,11 +139,11 @@ check_tag(session *ssn, int tag)
  * Check if server sent a BYE response (connection is closed immediately).
  */
 int
-check_bye(void)
+check_bye(char *buf)
 {
 
-	if (xstrcasestr(ibuf.data, "* BYE") &&
-	    !xstrcasestr(ibuf.data, " LOGOUT "))
+	if (xstrcasestr(buf, "* BYE") &&
+	    !xstrcasestr(buf, " LOGOUT "))
 		return 1;
 	else
 		return 0;
@@ -152,10 +154,10 @@ check_bye(void)
  * Check if the server sent a continuation request.
  */
 int
-check_continuation(void)
+check_continuation(char *buf)
 {
 
-	if (ibuf.data[0] == '+' && ibuf.data[1] == ' ')
+	if (buf[0] == '+' && buf[1] == ' ')
 		return 1;
 	else
 		return 0;
@@ -166,10 +168,10 @@ check_continuation(void)
  * Check if the server sent a TRYCREATE response.
  */
 int
-check_trycreate(void)
+check_trycreate(char *buf)
 {
 
-	if (xstrcasestr(ibuf.data, "[TRYCREATE]"))
+	if (xstrcasestr(buf, "[TRYCREATE]"))
 		return 1;
 	else
 		return 0;
@@ -183,6 +185,7 @@ int
 response_generic(session *ssn, int tag)
 {
 	int r;
+	ssize_t n;
 
 	if (tag == -1)
 		return -1;
@@ -190,17 +193,19 @@ response_generic(session *ssn, int tag)
 	buffer_reset(&ibuf);
 
 	do {
-		buffer_check(&ibuf, strlen(ibuf.data) + INPUT_BUF);
-		if (receive_response(ssn, ibuf.data + strlen(ibuf.data)) == -1)
+		buffer_check(&ibuf, ibuf.len + INPUT_BUF);
+		if ((n = receive_response(ssn, ibuf.data + ibuf.len)) == -1)
 			return -1;
-		if (check_bye())
+		ibuf.len += n;
+
+		if (check_bye(ibuf.data))
 			return -1;
-		if (check_continuation())
+		if (check_continuation(ibuf.data))
 			return STATUS_RESPONSE_CONTINUE;
-	} while ((r = check_tag(ssn, tag)) == STATUS_RESPONSE_NONE);
+	} while ((r = check_tag(ibuf.data, ssn, tag)) == STATUS_RESPONSE_NONE);
 
 	if (r == STATUS_RESPONSE_NO &&
-	    (check_trycreate() || get_option_boolean("create")))
+	    (check_trycreate(ibuf.data) || get_option_boolean("create")))
 		return STATUS_RESPONSE_TRYCREATE;
 
 	return r;
@@ -219,7 +224,7 @@ response_greeting(session *ssn)
 	if (receive_response(ssn, ibuf.data) == -1)
 		return -1;
 
-	if (check_bye())
+	if (check_bye(ibuf.data))
 		return -1;
 
 	verbose("S (%d): %s", ssn->socket, ibuf);
@@ -252,9 +257,9 @@ response_capability(session *ssn, int tag)
 		s = xstrndup(ibuf.data + re->pmatch[1].rm_so,
 		    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
 
-		if (xstrcasestr(ibuf.data, "IMAP4rev1"))
+		if (xstrcasestr(s, "IMAP4rev1"))
 			ssn->protocol = PROTOCOL_IMAP4REV1;
-		else if (xstrcasestr(ibuf.data, "IMAP4"))
+		else if (xstrcasestr(s, "IMAP4"))
 			ssn->protocol = PROTOCOL_IMAP4;
 		else {
 			error("server supports neither the IMAP4rev1 nor the "
@@ -264,17 +269,17 @@ response_capability(session *ssn, int tag)
 
 		ssn->capabilities = CAPABILITY_NONE;
 
-		if (xstrcasestr(ibuf.data, "NAMESPACE"))
+		if (xstrcasestr(s, "NAMESPACE"))
 			ssn->capabilities |= CAPABILITY_NAMESPACE;
 #ifndef NO_CRAMMD5
-		if (xstrcasestr(ibuf.data, "AUTH=CRAM-MD5"))
+		if (xstrcasestr(s, "AUTH=CRAM-MD5"))
 			ssn->capabilities |= CAPABILITY_CRAMMD5;
 #endif
 #ifndef NO_SSLTLS
-		if (xstrcasestr(ibuf.data, "STARTTLS"))
+		if (xstrcasestr(s, "STARTTLS"))
 			ssn->capabilities |= CAPABILITY_STARTTLS;
 #endif
-		if (xstrcasestr(ibuf.data, "CHILDREN"))
+		if (xstrcasestr(s, "CHILDREN"))
 			ssn->capabilities |= CAPABILITY_CHILDREN;
 
 		xfree(s);
@@ -420,7 +425,8 @@ response_select(session *ssn, int tag)
 
 
 /*
- * Process the data that server sent due to IMAP LIST client request.
+ * Process the data that server sent due to IMAP LIST or IMAP LSUB client
+ * request.
  */
 int
 response_list(session *ssn, int tag, char **mboxs, char **folders)
@@ -433,35 +439,36 @@ response_list(session *ssn, int tag, char **mboxs, char **folders)
 	if ((r = response_generic(ssn, tag)) == -1)
 		return -1;
 
-	m = *mboxs = (char *)xmalloc((strlen(ibuf.data) + 1) * sizeof(char));
-	f = *folders = (char *)xmalloc((strlen(ibuf.data) + 1) * sizeof(char));
+	m = *mboxs = (char *)xmalloc((ibuf.len + 1) * sizeof(char));
+	f = *folders = (char *)xmalloc((ibuf.len + 1) * sizeof(char));
 	*m = *f = '\0';
 
 	re = &responses[DATA_RESPONSE_LIST];
 
 	b = ibuf.data;
 	while (!regexec(re->preg, b, re->nmatch, re->pmatch, 0)) {
-		a = xstrndup(b + re->pmatch[1].rm_so,
-		    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
+		a = xstrndup(b + re->pmatch[2].rm_so,
+		    re->pmatch[2].rm_eo - re->pmatch[2].rm_so);
 
-		if (re->pmatch[4].rm_so != -1 && re->pmatch[4].rm_so != -1)
-			s = xstrndup(b + re->pmatch[4].rm_so,
-			    re->pmatch[4].rm_eo - re->pmatch[4].rm_so);
-		else if (re->pmatch[5].rm_so != -1 &&
-		    re->pmatch[5].rm_so != -1)
+		if (re->pmatch[5].rm_so != -1 && re->pmatch[5].rm_so != -1)
 			s = xstrndup(b + re->pmatch[5].rm_so,
 			    re->pmatch[5].rm_eo - re->pmatch[5].rm_so);
+		else if (re->pmatch[6].rm_so != -1 &&
+		    re->pmatch[6].rm_so != -1)
+			s = xstrndup(b + re->pmatch[6].rm_so,
+			    re->pmatch[6].rm_eo - re->pmatch[6].rm_so);
 		else
-			s = xstrndup(b + re->pmatch[7].rm_so, strtoul(b +
-			    re->pmatch[6].rm_so, NULL, 10));
+			s = xstrndup(b + re->pmatch[8].rm_so, strtoul(b +
+			    re->pmatch[7].rm_so, NULL, 10));
 
 		v = reverse_namespace(s, ssn->ns.prefix, ssn->ns.delim);
 		n = strlen(v);
 
 		if (!xstrcasestr(a, "\\NoSelect")) {
-			xstrncpy(m, v, n);
+			xstrncpy(m, v, ibuf.len - (m - *mboxs));
 			m += n;
-			xstrncpy(m++, "\n", 1);
+			xstrncpy(m, "\n", ibuf.len - (m - *mboxs));
+			m += strlen("\n");
 		}
 
 		if (!xstrcasestr(a, "\\NoInferiors") &&
@@ -469,74 +476,10 @@ response_list(session *ssn, int tag, char **mboxs, char **folders)
 		    ((ssn->capabilities & CAPABILITY_CHILDREN) &&
 		    (xstrcasestr(a, "\\HasChildren")) &&
 		    !xstrcasestr(a, "\\HasNoChildren")))) {
-			xstrncpy(f, v, n);
+			xstrncpy(f, v, ibuf.len - (f - *folders));
 			f += n;
-			xstrncpy(f++, "\n", 1);
-		}
-
-		b += re->pmatch[0].rm_eo;
-
-		xfree(a);
-		xfree(s);
-	}
-
-	return r;
-}
-
-
-/*
- * Process the data that server sent due to IMAP LSUB client request.
- */
-int
-response_lsub(session *ssn, int tag, char **mboxs, char **folders)
-{
-	int r, n;
-	char *b, *a, *s, *m, *f;
-	const char *v;
-	regexp *re;
-
-	if ((r = response_generic(ssn, tag)) == -1)
-		return -1;
-
-	m = *mboxs = (char *)xmalloc((strlen(ibuf.data) + 1) * sizeof(char));
-	f = *folders = (char *)xmalloc((strlen(ibuf.data) + 1) * sizeof(char));
-	*m = *f = '\0';
-
-	re = &responses[DATA_RESPONSE_LSUB];
-
-	b = ibuf.data;
-	while (!regexec(re->preg, b, re->nmatch, re->pmatch, 0)) {
-		a = xstrndup(b + re->pmatch[1].rm_so,
-		    re->pmatch[1].rm_eo - re->pmatch[1].rm_so);
-
-		if (re->pmatch[4].rm_so != -1 && re->pmatch[4].rm_so != -1)
-			s = xstrndup(b + re->pmatch[4].rm_so,
-			    re->pmatch[4].rm_eo - re->pmatch[4].rm_so);
-		else if (re->pmatch[5].rm_so != -1 &&
-		    re->pmatch[5].rm_so != -1)
-			s = xstrndup(b + re->pmatch[5].rm_so,
-			    re->pmatch[5].rm_eo - re->pmatch[5].rm_so);
-		else
-			s = xstrndup(b + re->pmatch[7].rm_so, strtoul(b +
-			    re->pmatch[6].rm_so, NULL, 10));
-
-		v = reverse_namespace(s, ssn->ns.prefix, ssn->ns.delim);
-		n = strlen(v);
-
-		if (!xstrcasestr(a, "\\NoSelect")) {
-			xstrncpy(m, v, n);
-			m += n;
-			xstrncpy(m++, "\n", 1);
-		}
-
-		if (!xstrcasestr(a, "\\NoInferiors") &&
-		    (!(ssn->capabilities & CAPABILITY_CHILDREN) ||
-		    ((ssn->capabilities & CAPABILITY_CHILDREN) &&
-		    (xstrcasestr(a, "\\HasChildren")) &&
-		    !xstrcasestr(a, "\\HasNoChildren")))) {
-			xstrncpy(f, v, n);
-			f += n;
-			xstrncpy(f++, "\n", 1);
+			xstrncpy(f, "\n", ibuf.len - (f - *folders));
+			f += strlen("\n");
 		}
 
 		b += re->pmatch[0].rm_eo;
@@ -555,7 +498,7 @@ response_lsub(session *ssn, int tag, char **mboxs, char **folders)
 int
 response_search(session *ssn, int tag, char **mesgs)
 {
-	int r;
+	int r, min;
 	regexp *re;
 	char *b, *m;
 
@@ -568,15 +511,19 @@ response_search(session *ssn, int tag, char **mesgs)
 	m = NULL;
 	while (!regexec(re->preg, b, re->nmatch, re->pmatch, 0)) {
 		if (!*mesgs) {
-			m = *mesgs = (char *)xmalloc((strlen(ibuf.data) + 1) *
+			m = *mesgs = (char *)xmalloc((ibuf.len + 1) *
 			    sizeof(char));
 			*m = '\0';
 		}
 
-		xstrncpy(m, b + re->pmatch[1].rm_so, re->pmatch[1].rm_eo -
-		    re->pmatch[1].rm_so);
-		m += re->pmatch[1].rm_eo - re->pmatch[1].rm_so;
-		xstrncpy(m++, " ", 1);
+		min = re->pmatch[1].rm_eo - re->pmatch[1].rm_so < ibuf.len ?
+		    re->pmatch[1].rm_eo - re->pmatch[1].rm_so :
+		    ibuf.len;
+
+		xstrncpy(m, b + re->pmatch[1].rm_so, min);
+		m += min;
+		xstrncpy(m++, " ", ibuf.len - min);
+		m += strlen(" ");
 
 		b += re->pmatch[0].rm_eo;
 	}
@@ -625,79 +572,59 @@ response_fetchfast(session *ssn, int tag, char **flags, char **date,
 
 
 /*
- * Process the data that server sent due to IMAP FETCH BODY[HEADER] client
- * request.
+ * Process the data that server sent due to IMAP FETCH BODY[] client request,
+ * ie. FETCH BODY[HEADER], FETCH BODY[TEXT], FETCH BODY[HEADER.FIELDS ()].
  */
 int
-response_fetchheader(session *ssn, int tag, char **header,
-    unsigned long int *len)
+response_fetchbody(session *ssn, int tag, char **body, size_t *len)
 {
-	int r;
+	int r, match, offset;
+	ssize_t n;
 	regexp *re;
 
-	if ((r = response_generic(ssn, tag)) == -1)
+	if (tag == -1)
 		return -1;
 
-	re = &responses[DATA_RESPONSE_FETCH_HEADER];
-	if (!regexec(re->preg, ibuf.data, re->nmatch, re->pmatch, 0)) { 
-		*header = ibuf.data + re->pmatch[2].rm_so;
-		*len = strtoul(ibuf.data + re->pmatch[1].rm_so, NULL, 10);
-	}
-
-	return r;
-}
-
-
-/*
- * Process the data that server sent due to IMAP FETCH BODY[TEXT] client
- * request.
- */
-int
-response_fetchbody(session *ssn, int tag, char **body,
-    unsigned long int *len)
-{
-	int r;
-	regexp *re;
-
-	if ((r = response_generic(ssn, tag)) == -1)
-		return -1;
+	buffer_reset(&ibuf);
 
+	match = -1;
+	offset = 0;
+	
 	re = &responses[DATA_RESPONSE_FETCH_BODY];
-	if (!regexec(re->preg, ibuf.data, re->nmatch, re->pmatch, 0)) { 
-		if (re->pmatch[2].rm_so != -1 && re->pmatch[2].rm_eo != -1) {
-			*body = ibuf.data + re->pmatch[3].rm_so;
-			*len = strtoul(ibuf.data + re->pmatch[2].rm_so, NULL,
-			    10);
-		} else {
 
-			*body = ibuf.data + re->pmatch[4].rm_so;
-			*len = re->pmatch[4].rm_eo - re->pmatch[4].rm_so;
+	do {
+		buffer_check(&ibuf, ibuf.len + INPUT_BUF);
+		if ((n = receive_response(ssn, ibuf.data + ibuf.len)) == -1)
+			return -1;
+		ibuf.len += n;
+
+		if (ibuf.len >= offset)
+			if (check_bye(ibuf.data + offset))
+				return -1;
+
+		if (match != 0) {
+			match = regexec(re->preg, ibuf.data, re->nmatch,
+			    re->pmatch, 0);
+			
+			if (match == 0 && re->pmatch[2].rm_so != -1 &&
+			    re->pmatch[2].rm_eo != -1) {
+				*len = strtoul(ibuf.data + re->pmatch[2].rm_so,
+				    NULL, 10);
+				offset = re->pmatch[0].rm_eo + *len;
+			}
+		}
+	} while (ibuf.len < offset || (r = check_tag(ibuf.data + offset, ssn,
+	    tag)) == STATUS_RESPONSE_NONE);
+
+	if (match == 0) {
+		if (re->pmatch[2].rm_so != -1 &&
+		    re->pmatch[2].rm_eo != -1) {
+			*body = ibuf.data + re->pmatch[0].rm_eo;
+		} else {
+			*body = ibuf.data + re->pmatch[3].rm_so;
+			*len = re->pmatch[3].rm_eo - re->pmatch[3].rm_so;
 		}
 	}
 
 	return r;
 }
-
-
-/*
- * Process the data that server sent due to IMAP FETCH BODY[HEADER.FIELDS ()]
- * client request.
- */
-int
-response_fetchfields(session *ssn, int tag, char **fields,
-    unsigned long int *len)
-{
-	int r;
-	regexp *re;
-
-	if ((r = response_generic(ssn, tag)) == -1)
-		return -1;
-
-	re = &responses[DATA_RESPONSE_FETCH_FIELDS];
-	if (!regexec(re->preg, ibuf.data, re->nmatch, re->pmatch, 0)) { 
-		*fields = ibuf.data + re->pmatch[2].rm_so;
-		*len = strtoul(ibuf.data + re->pmatch[1].rm_so, NULL, 10);
-	}
-
-	return r;
-}
