@@ -110,10 +110,10 @@ delete(myaccount, 'mymailbox', results)
 -- it considered the message "spam" and 0 otherwise:
  
 results = match(myaccount, 'mymailbox', {})
-text = fetchtext(myaccount, 'mymailbox', results)
+messages = fetchmessage(myaccount, 'mymailbox', results)
 
 results = {}
-for msgid, msgtxt in pairs(text) do
+for msgid, msgtxt in pairs(messages) do
     if (pipe_to('bayesian-spam-filter', msgtxt) == 1) then
 	table.insert(results, msgid)
     end
